from argparse import Namespace
from mock import patch
from StringIO import StringIO
from tempfile import NamedTemporaryFile
from unittest import TestCase

from yaml import safe_dump

from workspace_runner import (
    parse_args,
    workspace_run,
    Runner,
    SSHRunner,
    )


class TestParseArgs(TestCase):

    def test_minimal(self):
        args = parse_args(['foo', 'bar'])
        self.assertEqual(args, Namespace(config='foo', host='bar'))


class FakeRunner(Runner):
    """"Class that pretends to run commands."""

    last_instance = None

    @classmethod
    def factory(cls, host):
        instance = cls(host)
        cls.last_instance = instance
        return instance

    def __init__(self, host):
        super(FakeRunner, self).__init__(host)
        self.run_calls = []

    def run(self, args, out_file):
        self.run_calls.append(args)
        out_file.write(''.join(args))


class TestRunnerClass:

    def test_host(self):
        runner = self.runner_class('asdf')
        self.assertEqual(runner.host, 'asdf')

    def test_run(self):
        runner = self.runner_class('asdf')
        out_file = StringIO()
        self.expect_run(runner, ['foo', 'bar'], out_file)
        self.assertEqual(out_file.getvalue(), 'foobar')


class TestSSHRunner(TestRunnerClass, TestCase):

    runner_class = SSHRunner

    def expect_run(self, runner, args, out_file):
        def effect(args, stdout):
            stdout.write(''.join(args[2:]))

        with patch('subprocess.check_call', side_effect=effect,
                   autospec=True) as cc_mock:
            runner.run(args, out_file)
        cc_mock.assert_called_once_with(['ssh', 'asdf'] + args,
                                        stdout=out_file)


class TestFakeRunner(TestRunnerClass, TestCase):

    runner_class = FakeRunner

    def expect_run(self, runner, args, out_file):
        runner.run(args, out_file)
        self.assertEqual(runner.run_calls, [['foo', 'bar']])


class TestWorkspaceRun(TestCase):

    def test_minimal(self):
        with NamedTemporaryFile() as config_file:
            safe_dump({'command': ['run', 'this']}, config_file)
            workspace_run([config_file.name, 'bar'], FakeRunner.factory)
        runner = FakeRunner.last_instance
        self.assertIsNot(runner, None)
        self.assertEqual(runner.run_calls, [['run', 'this']])
